/* Open SoundControl kit in C++                                              */
/* Copyright (C) 2002 Daniel Holth, Clinton McChesney                        */
/*                                                                           */
/* This library is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU Lesser General Public                */
/* License as published by the Free Software Foundation; either              */
/* version 2.1 of the License, or (at your option) any later version.        */
/*                                                                           */
/* This library is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         */
/* Lesser General Public License for more details.                           */
/*                                                                           */
/* You should have received a copy of the GNU Lesser General Public          */
/* License along with this library; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                           */
/* For questions regarding this program contact                              */
/* Daniel Holth <dholth@stetson.edu> or visit                                */
/* http://www.stetson.edu/~ProctoLogic/                                      */

/**
 * @file Broadcaster.cc
 *
 * Sends a message to n subscribed hosts.  
 *
 * @author Daniel Holth
 * @author Clinton McChesney
 * @author Martin Rumori
 */

#include "Broadcaster.h"
#include <sstream>

using namespace std;

void Broadcaster::addSubscriber(Transmit *subscriber)
{
  string newName;
  string oldName;
  TransmitList* tList;
  TransmitList::iterator  iterator;
  //  TransmitList::iterator  end; // not used
  bool   contains;
  time_t expiration;
  time_t leaseLength;
  
  // this is polymorphic - copy constructors are not.
  tList       = this->getSubscribers();
  contains    = false;
  newName     = subscriber->getName();
  expiration  = time(NULL);
  leaseLength = this->getLeaseLength();
  expiration  = expiration + leaseLength;

  for(iterator = tList->begin(); iterator != tList->end(); iterator++) 
  {
    oldName = (iterator->second)->getName();
    if(oldName == newName)
    {
      contains = true;
      break;
    }
  }
  
  if(contains == false)
  {
    subscriber = subscriber->clone();
    tList->push_back(RankedTransmit(expiration,subscriber));
#ifdef LIBOSC_DEBUG
    cout << "I (" << this->getName() << ") am adding: " << newName << endl;
#else
	cout << "Adding " << newName << endl;
#endif
  }
  else
  {
    cout << oldName << " was already in list, updating subscription." << endl;
    iterator->first = expiration;
  }
}

void Broadcaster::removeSubscriber(Transmit *subscriber)
{
  string newName;
  string oldName;
  TransmitList* tList;
  TransmitList::iterator  iterator;
  //   TransmitList::iterator  end; // not used
  bool   contains;

  subscriber  = subscriber->clone();
  
  tList       = this->getSubscribers();
  iterator    = tList->begin();
  //   end         = tList->end(); // not used
  contains    = false;
  newName     = subscriber->getName();

  for(iterator = tList->begin(); iterator != tList->end(); iterator++) 
  {
    oldName = (iterator->second)->getName();
    if(oldName == newName)
    {
      contains = true;
      break;
    }
  }
  
  if(contains == true)
  {
    cout << "Removing subscription for " << oldName <<  "." << endl;
    delete subscriber;
    tList->erase(iterator);
  }
}
 
bool Broadcaster::transmitInternal(Transmit* destination, const string *data)
{
  return destination->transmit(*data);
}


bool Broadcaster::transmit(const string &data)
{
  TransmitList::iterator  iterator;
  TransmitList::iterator  end;
  TransmitList* tList;
  Transmit*     transmit;
  bool   shouldKeep;
  time_t expiration;
  time_t now;
  
  tList      = this->getSubscribers();
  iterator   = tList->begin();
  end        = tList->end();
  shouldKeep = false;
  now        = time(NULL);
 
#ifdef LIBOSC_DEBUG
  cout << "I'm transmitting!  Really! " << endl;
#endif
  
  while(iterator != end)
  {
    transmit   = iterator->second;
    expiration = iterator->first;
#ifdef LIBOSC_DEBUG
    cout << "Transmitting to: " << transmit->getName() 
         << ", Expires in: "    << (expiration - now) << endl;
#endif
    if(now <= expiration)
    {
      shouldKeep = this->transmitInternal(transmit, &data);
    }
    
    if(shouldKeep == false)
    {
      cout << "Problem transmitting to: " << transmit->getName() << endl;
      delete transmit;
      iterator = tList->erase(iterator);
    }
    else
    {
      iterator++;
    }
  }
#ifdef LIBOSC_DEBUG
  cout << "I'm done transmitting. " << endl;
#endif
  return true;
}

std::string Broadcaster::getName() {
	ostringstream name;
	name << "Broadcaster@" << this;
	return name.str();
}
